<?php
namespace Models;

// Модель для настроек
class Settings extends Controller {
	public $db, $f3;

	public function __construct($f3, $db=null) {
		parent::__construct($f3);
		$this->db=$db;
		$this->f3=$f3;
	}

	// данные о страницах
	function page_data($page='api') {
		$pages=array(
			'api'=>array(
				'title'=>'Настройки подключения к API',
				'inc'=>'settings_api.htm'
			),
			'synchronization'=>array(
				'title'=>'Синхронизация с Агрота',
				'inc'=>'settings_synchronization.htm'
			),
			'kkm'=>array(
				'title'=>'Настройка Контрольно-кассовой машины',
				'inc'=>'settings_fr.htm'
			),
			'logs'=>array(
				'title'=>'Ошибки cинхронизации',
				'inc'=>'settings_logs.htm'
			),
			'cd'=>array(
				'title'=>'Настройка дисплея покупателя',
				'inc'=>'settings_cd.htm'
			),
			'service'=>array(
				'title'=>'Сервисное обслуживание',
				'inc'=>'settings_service.htm'
			),
			'base'=>array(
				'title'=>'Основные настройки',
				'inc'=>'settings_base.htm'
			),
			'rooms'=>array(
				'title'=>'Управление залами',
				'inc'=>'settings_rooms.htm'
			),
			'drivers'=>array(
				'title'=>'Настройка драйвера',
				'inc'=>'settings_drivers.htm'
			),
			'update_ticket_time'=>array(
				'title'=>'Изменить время выгрузки чеков',
				'inc'=>'settings_update_ticket_time.htm'
			),
			'scales'=>array(
				'title'=>'Настройка весов',
				'inc'=>'settings_scales.htm'
			),
			'kitchen_printer'=>array(
				'title'=>'Настройка принтера для кухни',
				'inc'=>'settings_kitchen_printer.htm'
			),
			'pre_printer'=>array(
				'title'=>'Настройка принтера для пречеков',
				'inc'=>'settings_pre_printer.htm'
			),
			'bar_printer'=>array(
				'title'=>'Настройка принтера для бара',
				'inc'=>'settings_bar_printer.htm'
			),
			'autonomous'=>array(
				'title'=>'Настройка автономной работы',
				'inc'=>'settings_autonomous.htm'
			),
			'productcategories'=>array(
				'title'=>'Справочник категорий номенклатуры',
				'inc'=>'settings_productcategories.htm'
			)
		);
		$pages[$page]['page']=$page;

		return $pages[$page];
	}

	// Получаем настройки из таблицы настроек
	function get_settings($name=array()) {
		$sets=array();
		if($name) {
			$sql="SELECT * FROM `settings` WHERE ";
			$where='';
			$val=array();
			foreach ($name as $n) {
				if($where!='')$where.=' OR ';
				$where.="`name`=?";
				$val[]=$n;
			}
			if($where!='') {
				$s=$this->db->exec($sql.$where, $val);
				if($s) {
					foreach ($s as $val) {
						$sets[$val['name']]=$val['value'];
					}
				}
			}
		}
		return $sets;
	}

	// Получаем основные настройки программы
	function get_base() {
		$name=array(
			'auto_fullscreen',
			'virtual_keyboard',
			'descriptions_pattern',
			'cleaning',
			'auto_logout',
			'cafe_mode',
			'auto_fiscalization',
			'timezone',
			'auto_replacekey',
			'check_mark',
			'check_mrc',
			'egais',
			'check_egais',
			'url_egais',
			'product_menu',
			'view_all_receipts',
			'lines_of_list',
			'modifiers_window',
			'dark_theme',
			'round_kop',
			'executor',
			'balance',
			'multikassa',
			'ip_range_start',
			'ip_range_end',
			'other_cashboxes',
			'autonomous'
		);
		$sets=$this->get_settings($name);
		if($sets['other_cashboxes']) {
			$sets['other_cashboxes']=json_decode($sets['other_cashboxes'], true);
			$sets['other_cashboxes_ip']=array_keys($sets['other_cashboxes']);
		}
		return $sets;
	}

	// Сохраняем основные настройки программы
	function save_base($post) {
		if(isset($post['auto_logout']) && !is_numeric($post['auto_logout']) || $post['auto_logout']<0) {
			return array(
				'success'=>false,
				'txt'=>'incorrect_auto_logout'
			);
		}
		if(isset($post['egais']) && $post['egais']>0 && $post['url_egais']=='') {
			return array(
				'success'=>false,
				'txt'=>'incorrect_egais_url'
			);
		}
		$sql="INSERT OR REPLACE INTO `settings` (`name`, `value`) VALUES ";
		$val=array();
		foreach ($post as $key => $value) {
			if(!empty($val)) {
				$sql.=", ";
			}
			$sql.="(?, ?)";
			$val[]=$key;
			$val[]=$value;
		}
		$res=$this->db->exec($sql, $val);
		if($res!==false) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload_menu'=>true
			);
		}
		return array(
			'success'=>false,
			'txt'=>'db_error'
		);
	}

	// Создаем файл настройек API
	function create_config_api($data) {
		$txt="<?php\r\n";
		$txt.='$api_store_id="'.$data['store_id'].'";'."\n\r";
		$txt.='$api_rm_id="'.$data['rm_id'].'";'."\n\r";
		$txt.='$api_domen="'.$data['domen'].'";'."\n\r";
		$txt.='$api_pass="'.$data['pass'].'";'."\n\r";
		$txt.='$api_count="'.$data['count'].'";'."\n\r";
		$this->f3->write('app/api_config.php', $txt);
	}
	
	// Сохраняем настройки API
	function post_api($post) {
		if(!$post['domen'] || !$post['pass'] || !$post['store_id'] || !$post['rm_id'] || !$post['count']) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}
		$api=new \Models\Api($this->f3, $post['domen'], $post['pass'], $post['store_id'], $post['count']);
		
		$res=$api->users();
		if(!$res['success']) {
			$txt='';
			$m=new \Models\Messages();
			foreach ($res['error'] as $er) {
				$txt.=$m->message($er)['txt'].'<br>';
			}
			return array(
				'success'=>false,
				'txt'=>$txt,
				'debug'=>$res['debug']
			);
		}
		if(count($res['users'])<=0) {
			return array(
				'success'=>false,
				'txt'=>'cashbox_users_not_found'
			);
		}
		$this->create_config_api($post);
		$res=array(
			'success'=>true,
			'txt'=>'save_successful',
			'reroute'=>'/'
		);

		if($this->f3->get('authorization_user')) {
			unset($res['reroute']);
		}
		return $res;
	}

	// Сохраняем настройки автономного режима
	function save_autonomous($data) {
		$error=0;
		if($data['autonomous']) {
			$res=$this->db->exec("UPDATE fr_settings SET rm=?, sno=?", array((string)$data['rm_id'], $data['sno']));
			if($res===false) {
				$error++;
			}
			$res=$this->db->exec("UPDATE cd_settings SET rm=?", array((string)$data['rm_id']));
			if($res===false) {
				$error++;
			}
			$res=$this->db->exec("UPDATE scales_settings SET rm=?", array((string)$data['rm_id']));
			if($res===false) {
				$error++;
			}
			foreach ($data['barcodes_pr_api'] as $name => $value) {
				$sql[]="INSERT OR REPLACE INTO settings (name, value) VALUES (?, ?)";
				$val[]=array($name, $value);
			}
		}
		$sql[]="INSERT OR REPLACE INTO settings (name, value) VALUES (?, ?)";
		$val[]=array('autonomous', $data['autonomous']);
		$res=$this->db->exec($sql, $val);
		if($res===false) {
			$error++;
		}
		if($error>0) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reload_menu'=>true
		);
	}

	// Сохраняем настройки автосинхронизации
	function post_synchronization($db,$auto_sinch) {
		$res=$db->exec("UPDATE synchronization SET auto_sinch=?", $auto_sinch);
		if($res>=0) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.auto_sinch_info',
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// сброс всех настроек и удаление базы данных
	function reset() {
		if(!@unlink('app/api_config.php')) {
			return array(
				'success'=>false,
				'txt'=>'file_is_not_delete'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reroute'=>'/'
		);
	}

	// Удаление старых чеков
	function cleaning() {
		// Узнаем чеки для удаления и формируем данные для запроса удаления строк
		$sql="SELECT id FROM `receipts` WHERE time<=? AND time>0";
		$time=time()-3600*24*60;
		$tickets=$this->db->exec($sql, $time);
		$in='';
		$val=array();
		if($tickets) {
			foreach ($tickets as $t) {
				if($in!='')$in.=', ';
				$in.='?';
				$val[]=$t['id'];
			}
		}
		if($in!='') {
			// Удаляем чеки
			$sql="DELETE FROM `receipts` WHERE `id` IN (".$in.")";
			if($this->db->exec($sql, $val)!==false) {
				$sql="DELETE FROM `receiptlines` WHERE `receipt_id` IN (".$in.")";
				$this->db->exec($sql, $val);
			}
		}
		$this->save_base(array('cleaning'=>time()));
		$this->db->exec("VACUUM");
	}

	// Сохраненеи настроек драйвера
	function save_driver($data) {
		$file=$data['path'].'/server.config';
		if(!file_exists($file)) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		// Сохранение изначального файла настроек, при первом изменении
		$file_default=$file.'.default';
		if(!file_exists($file_default)) {
			copy($file, $file_default);
		}
		if($data['device']!='pre' && $data['device']!='kitchen' && $data['device']!='bar') {
			$data['device']='';
		}
		if($data['device']!='') {
			$file=$data['path'].'/'.$data['device'].'_server.config';
			if(!file_exists($file_default)) {
				copy($file_default, $file);
			}
		}
		if(file_put_contents($file, $data['txt'])) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'error'
			);
		}
	}

	// Запуск драйвера
	function start_driver($driver) {
		$script=$driver.'/server.py';
		$folders=explode('/', $driver);
		if(!file_exists($script)) {// Если файла нет, то это нативный драйвер
			if($folders[1]=='printers') {
				$sets=$this->db->exec("SELECT * FROM  `fr_settings` LIMIT 1")[0];
			}
			else {
				$sets=$this->db->exec("SELECT * FROM  `cd_settings` LIMIT 1")[0];
			}
			$sets['driver']=$driver;
			return $this->start_driver_php($sets);
			/*return array(
				'success'=>false,
				'txt'=>'not_found'
			);*/
		}
		include('drivers/start.php');

		// Проверяем запуск драйвера
		$models=array(
			'printers'=>'\Models\Fr',
			'displays'=>'\Models\Cd'
		);
		$m=new $models[$folders[1]]($this->db, $this->f3);
		//$m->test($driver);

		return array(
			'success'=>true,
			'txt'=>'success'
		);
	}

	function start_driver_php($sets, $device='kkm') {
		$script=$sets['driver'].'/server.php';
		if($device=='kitchen' || $device=='pre' || $device=='bar') {
			$script=$sets['driver'].'/'.$device.'server.php';
		}
		if(!file_exists($script)) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}

		$folders=explode('/', $sets['driver']);
		if($folders[1]=='printers') {
			include($sets['driver'].'/server.php');
			$class=array_reverse($folders)[0];
			$fr_model='\Models\\'.$class.'_Fr_php';
			$config_file='/server.config';
			if($device=='pre' || $device=='kitchen' || $device=='bar') {
				if(file_exists($sets['driver'].'/'.$device.'_server.config')) {
					$config_file='/'.$device.'_server.config';
				}
			}
			$config=parse_ini_file($sets['driver'].$config_file);
			$m=new $fr_model($this->f3, $sets, $config);
			$m->kkm_init();
		}
		else {
			if(!class_exists('Models\Cd_php', false)) {
				include($sets['driver'].'/server.php');
			}
			$m=new Cd_php($this->f3, $sets);
			$m->cd_init();
		}
		return array(
			'success'=>true,
			'txt'=>'success'
		);
	}

	// Сохранение изменения времени выгрузки чеков
	function save_ticket_time($db, $data) {
		$data=strtotime($data);
		if(!is_numeric($data)) {
			return array(
				'success'=>false,
				'txt'=>'incorrect_data'
			);
		}

		$res=$db->exec("UPDATE synchronization SET tickets_update=?", $data);
		if($res!==false) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.ticket_time',
				'close_all_window'=>true
			);
		}
		return array(
			'success'=>false,
			'txt'=>'db_error'
		);
	}
	
	function get_timezones() {
		$timezones=array(
			'Europe/Kaliningrad'=>'Калининград (МСК -1)',
			'Europe/Moscow'=>'Москва (МСК)',
			'Europe/Kirov'=>'Киров (МСК)',
			'Europe/Simferopol'=>'Симферополь (МСК)',
			'Europe/Volgograd'=>'Волгоград (МСК)',
			'Europe/Ulyanovsk'=>'Ульяновск (МСК +1)',
			'Europe/Saratov'=>'Саратов (МСК +1)',
			'Europe/Samara'=>'Самара (МСК +1)',
			'Asia/Yekaterinburg'=>'Екатеринбург (МСК +2)',
			'Asia/Omsk'=>'Омск (МСК +3)',
			'Europe/Astrakhan'=>'Астрахань (МСК +4)',
			'Asia/Barnaul'=>'Барнаул (МСК +4)',
			'Asia/Krasnoyarsk'=>'Красноярск (МСК +4)',
			'Asia/Novokuznetsk'=>'Новокузнецк (МСК +4)',
			'Asia/Tomsk'=>'Томск (МСК +4)',
			'Asia/Novosibirsk'=>'Новосибирск (МСК +4)',
			'Asia/Irkutsk'=>'Иркутск (МСК +5)',
			'Asia/Chita'=>'Чита (МСК +6)',
			'Asia/Yakutsk'=>'Якутск (МСК +6)',
			'Asia/Ust-Nera'=>'Усть-Нера (МСК +7)',
			'Asia/Vladivostok'=>'Владивосток (МСК +7)',
			'Asia/Magadan'=>'Магадан (МСК +8)',
			'Asia/Sakhalin'=>'Сахалин (МСК +8)',
			'Asia/Srednekolymsk'=>'Среднеколымск (МСК +8)',
			'Asia/Anadyr'=>'Анадырь (МСК +9)',
			'Asia/Kamchatka'=>'Камчатка (МСК +9)'
		);
		$month=date('n');
		$day=date('j');
		if(($month>7 && $month<2) || ($month==7 && $day>=17) || ($month==2 && $day<=2)) {
			$timezones['Europe/Volgograd']='Сталинград';
		}

		return $timezones;
	}
}
